<?php
/**
 * Connect shutdown notice.
 *
 * @package Flexible Shipping
 */

use WPDesk\Notice\Notice;
use WPDesk\PluginBuilder\Plugin\Hookable;

/**
 * Shows connect shutdown notice.
 */
class Flexible_Shipping_Connect_Shutdown_Notice implements Hookable {

	/**
	 * Connect connection.
	 *
	 * @var WPDesk_Flexible_Shipping_SaaS_Connection
	 */
	private $connection;

	/**
	 * Flexible_Shipping_Connect_Shutdown_Notice constructor.
	 *
	 * @param WPDesk_Flexible_Shipping_SaaS_Connection $connection .
	 */
	public function __construct( WPDesk_Flexible_Shipping_SaaS_Connection $connection ) {
		$this->connection = $connection;
	}

	/**
	 * Init hooks (actions and filters).
	 */
	public function hooks() {
		add_action( 'admin_init', array( $this, 'maybe_show_connect_notice' ) );
	}

	/**
	 * Maybe show connection shutdown notice.
	 */
	public function maybe_show_connect_notice() {
		if ( $this->connection->is_connected() ) {
			new Notice(
				sprintf(
					// Translators: link.
					__( 'Notice! We will shut down Connect on 22 August 2019! %1$sRead more →%2$s', 'flexible-shipping' ),
					'<a href="https://wpde.sk/connect-shut-down" target="_blank">',
					'</a>'
				),
				Notice::NOTICE_TYPE_ERROR
			);
		}
	}

}
